﻿Type=Class
Version=6.5
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
' This class REQUIRES:
'    o BluetoothAdmin
'    o Serial
Sub Class_Globals
	
	Public PageWidth2 As Int = 384 ' 192 dpi
	Public PageWidth3 As Int = 576 ' 192 dpi
	Public PageWidth4 As Int = 832 ' 208 dpi (maybe this sould be 768)

	
	Private BTAdmin As BluetoothAdmin
	Private BTDevices As Map
	
	Private DeviceName As String
	Private DeviceMAC As String
	
	Private eventPrefix As String
	Private eventModule As Object
	
	Private Ser As Serial

	Private SOS As OutputStream
	Private SIS As InputStream

	Private Ready As Boolean
	Private PageMode As Boolean

	Private CR As String = Chr(13)
	Private LF As String = Chr(10)
	Private FF As String = Chr(12)
	Private ESC As String = Chr(27)
	Private GS As String = Chr(29)
	
	Private csu As CallSubUtils

End Sub


'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize(Module As Object, EventName As String)
	
	eventModule = Module
	eventPrefix = EventName
	
	Log("Initializing BTAdmin from Star_SMT300i_Printer")
	BTAdmin.Initialize("BTAdmin")
	
	csu.Initialize
	
	Ready = False
End Sub

' **************************************************************
' ********************** Internal Methods **********************
' **************************************************************

Private Sub BTAdmin_StateChange(NewState As Int, OldState As Int)
	Log("BTAdmin_StateChange(NewState=" & NewState & ", OldState=" & OldState & ")")
End Sub

Private Sub BTAdmin_DiscoveryStarted
	Log("BTAdmin_DiscoveryStarted")
	BTDevices.Initialize
End Sub

Private Sub BTAdmin_DeviceFound( Name As String, MacAddress As String )
	Log("BTAdmin_DeviceFound(Name=""" & Name & """, MacAddress=""" & MacAddress & """)")
	BTDevices.Put(Name,MacAddress)
End Sub

Private Sub BTAdmin_DiscoveryFinished
	Log("BTAdmin_DiscoveryFinished")
	Log("   - discovered " & BTDevices.Size & " devices")
	Log("   - raising " & eventPrefix & "_UpdatedDevices event to caller.")
	Raise_UpdatedDevices
End Sub

Private Sub Ser_Connected(Success As Boolean)
	Log("Ser_Connected(Success=" & Success & ")")
	Ready = Success
	If Not(Success) Then
		Raise_Connected(Success)
	End If
	SOS = Ser.OutputStream
	SIS = Ser.InputStream
	Raise_Connected(Success)
End Sub


Private Sub SleepUntil( se As Long )
	Do While (DateTime.now < se)
		DoEvents
	Loop
End Sub

Public Sub WriteBytes( buf() As Byte )
	SOS.WriteBytes(buf,0,buf.Length)
End Sub

Private Sub ListToBytes(list As List) As Byte()
	Dim b(list.Size) As Byte
	For i = 0 To list.Size - 1
		b(i) = list.Get(i)
	Next
	Return b
End Sub

Private Sub FloatToL( value As Float ) As Byte
	Dim pixels As Int = value * 203 ' 192
	Return pixels Mod 256
End Sub

Private Sub FloatToH( value As Float ) As Byte
	Dim pixels As Int = value * 203' 192
	Return pixels / 256
End Sub


' **************************************************************
' ************ In support of events for our callers ************
' **************************************************************

Private Sub Raise_UpdatedDevices()
	CallSub2(eventModule,eventPrefix & "_UpdatedDevices",BTDevices)
End Sub

Private Sub Raise_Connected(Success As Boolean)
	CallSub2(eventModule,eventPrefix & "_Connected",Success)
End Sub


' **************************************************************
' ******************** Public Method Calls *********************
' **************************************************************

Public Sub RefreshDevices As Boolean
	If BTAdmin.StartDiscovery Then
		Log("   - StartDiscovery success")
		Return True
	End If
	Log("   - StartDiscovery failed")
	Return False
End Sub

Public Sub Open( Name As String, MacAddress As String )
	DeviceName = Name
	DeviceMAC = MacAddress
	
	Ser.Initialize("Ser")
	Ser.Connect(MacAddress)
End Sub

Public Sub Close() As Boolean
	Ready = False
	SOS.Flush
	' If we close before the printer has finished printing
	' what is in the buffer, then it will abort printing
	' before it is done. Introducing a delay here gives
	' the printer the chance to fnish up first.
	csu.CallSubDelayedPlus(Me,"CompleteClose",2000)
	Return True
End Sub

Private Sub CompleteClose()
	SOS.Close
	SIS.Close
	Ser.Disconnect	
End Sub

Public Sub WriteString( text As String )
	WriteBytes(text.GetBytes("UTF8"))
End Sub

Public Sub WriteLineString( text As String )
	WriteBytes((text & CR & LF ).GetBytes("UTF8"))
End Sub

Public Sub SetPageMode( WidthInches As Float, HeightInches As Float)
	Dim xL As Byte
	Dim xH As Byte
	Dim yL As Byte
	Dim yH As Byte
	Dim dxL As Byte
	Dim dxH As Byte
	Dim dyL As Byte
	Dim dyH As Byte
	
	xL = 0
	xH = 0
	yL = 0
	yH = 0
	dxL = FloatToL(WidthInches)
	dxH = FloatToH(WidthInches)
	dyL = FloatToL(HeightInches)
	dyH = FloatToH(HeightInches)
	
	Dim lst As List
	lst.Initialize
	lst.Add(Asc(ESC))
	lst.Add(Asc("W"))
	lst.Add(xL)
	lst.Add(xH)
	lst.Add(yL)
	lst.Add(yH)
	lst.Add(dxL)
	lst.Add(dxH)
	lst.Add(dyL)
	lst.Add(dyH)
	
	' Set page size
	WriteBytes(ListToBytes(lst))
	
	' Set page mode
	WriteBytes((ESC & "L").GetBytes("UTF8"))
	PageMode = True
End Sub

Public Sub FlushPageModePage
	WriteBytes((ESC & FF).GetBytes("UTF8"))	
End Sub

Public Sub SetStandardMode
	WriteBytes((ESC & "S").GetBytes("UTF8"))
	PageMode = False
End Sub

Public Sub SetHorizontalPosition( hPos As Float )
	Dim lst As List
	lst.Initialize
	lst.Add(Asc(ESC))
	lst.Add(Asc("$"))
	lst.Add(FloatToL(hPos))
	lst.Add(FloatToH(hPos))
	WriteBytes(ListToBytes(lst))
End Sub

Public Sub SetVerticalPostion( vPos As Float )
	Dim lst As List
	lst.Initialize
	lst.Add(Asc(GS))
	lst.Add(Asc("$"))
	lst.Add(FloatToL(vPos))
	lst.Add(FloatToH(vPos))
	WriteBytes(ListToBytes(lst))
End Sub

Public Sub SetPosition( hPos As Float, vPos As Float )
'	Dim lst As List
'	lst.Initialize
'	lst.Add(Asc(GS))
'	lst.Add(Asc("$"))
'	lst.Add(FloatToL(hPos))
'	lst.Add(FloatToH(hPos))
'	lst.Add(FloatToL(vPos))
'	lst.Add(FloatToH(vPos))
'	WriteBytes(ListToBytes(lst))
	SetVerticalPostion(vPos)
	SetHorizontalPosition(hPos)
End Sub

pUBLIC Sub SetEmphasize( on As Boolean )
	Dim lst As List
	lst.Initialize
	lst.Add(Asc(ESC))
	lst.Add(Asc("E"))
	If on Then
		lst.Add(1)
	Else
		lst.Add(0)
	End If
	WriteBytes(ListToBytes(lst))	
End Sub

Public Sub InitializePrinter
	WriteBytes((ESC & "@").GetBytes("UTF8"))	
End Sub

private Sub AddBitmapPixels( lst As List, bm As Bitmap )
	Dim x,y,w,h,sx,bpl As Int
	Dim stroke As Int

	w = bm.Width
	h = bm.Height
	bpl = (w + 7)/8
		
	y = 0
	x = 0
	Do While x < w
		sx = x
		For y = 0 To h-1
			stroke = 0
			x = sx
			If x < w Then
				If bm.GetPixel(x,y) = Colors.Black Then stroke = Bit.Or( stroke,128)
				x = x + 1
				If x < w Then
					If bm.GetPixel(x,y) = Colors.Black Then stroke = Bit.Or( stroke,64)
					x = x + 1
					If x < w Then
						If bm.GetPixel(x,y) = Colors.Black Then stroke = Bit.Or( stroke,32)
						x = x + 1
						If x < w Then
							If bm.GetPixel(x,y) = Colors.Black Then stroke = Bit.Or( stroke,16)
							x = x + 1
							If x < w Then
								If bm.GetPixel(x,y) = Colors.Black Then stroke = Bit.Or( stroke,8)
								x = x + 1
								If x < w Then
									If bm.GetPixel(x,y) = Colors.Black Then stroke = Bit.Or( stroke,4)
									x = x + 1
									If x < w Then
										If bm.GetPixel(x,y) = Colors.Black Then stroke = Bit.Or( stroke,2)
										x = x + 1
										If x < w Then
											If bm.GetPixel(x,y) = Colors.Black Then stroke = Bit.Or( stroke,1)
											x = x + 1
										End If
									End If
								End If
							End If
						End If
					End If
				End If
			End If
			lst.Add(stroke)
		Next
	Loop
	
End Sub

Public Sub WriteBitmap( bm As Bitmap)
	Dim w,h,bpl As Int
	Dim lst As List

	w = bm.Width
	h = bm.Height
	bpl = (w + 7)/8
	
	lst.Initialize

	lst.Add(Asc(ESC))
	lst.Add(Asc("X"))
	lst.Add(Asc("4"))
	lst.Add(bpl)
	lst.Add(h)
	
	AddBitmapPixels(lst,bm)

	'lst.Add(Asc(LF))
	lst.Add(Asc(ESC))
	lst.Add(Asc("X"))
	lst.Add(Asc("2"))
	lst.Add(bpl)
		
	WriteBytes(ListToBytes(lst))

End Sub

Public Sub WriteBitmap2( bm As Bitmap)
	Dim w,h,bpl As Int
	Dim lst As List

	w = bm.Width
	h = bm.Height
	bpl = (w + 7)/8
	
	lst.Initialize

	lst.Add(Asc(ESC))
	lst.Add(Asc("*"))
	lst.Add(33)
	lst.Add(FloatToL(w))
	lst.Add(FloatToH(w))
	
	AddBitmapPixels(lst,bm)

	lst.Add(Asc(LF))
	'lst.Add(Asc(ESC))
	'lst.Add(Asc("X"))
	'lst.Add(Asc("2"))
	'lst.Add(h)
		
	WriteBytes(ListToBytes(lst))

End Sub

Public Sub WriteBitmap3( bm As Bitmap )
	Dim offset,w,h,bpl,slice,b,k,x,y As Int
	Dim lst As List
	lst.Initialize

	w = bm.Width
	h = bm.Height
	bpl = (w + 7)/8

	lst.Add(Asc(ESC))
	lst.Add(Asc("@"))
	
	lst.Add(Asc(ESC))
	lst.Add(Asc("3"))
	lst.Add(Asc(24))

	offset = 0
	Do While offset < h
		lst.Add(Asc(ESC))
		lst.Add(Asc("*"))
		lst.Add(33)
		lst.Add(FloatToL(w))
		lst.Add(FloatToH(w))
		
		For x = 0 To w-1
			For k = 0 To 2
				slice = 0
				For b = 0 To 7
					y = (((offset/8)+k)*8)+b
					If y < w Then
						If bm.GetPixel(x,y) = Colors.Black Then
							Dim m As Int = Bit.ShiftLeft(1,7-b)
							slice = Bit.Or( slice, m )
						End If
					End If
				Next
				lst.Add(slice)
			Next
		Next
		offset = offset + 24
		lst.Add(10)
	Loop

	lst.Add(Asc(ESC))
	lst.Add(Asc("3"))
	lst.Add(Asc(30))

	WriteBytes(ListToBytes(lst))
	
End Sub
